import { Collection, Entity, Enum, ManyToOne, OneToMany, PrimaryKey, Property } from '@mikro-orm/core'
import { BaseEntity } from '#application/bases/baseEntity'
import { Sprite } from './sprite'
import { CharacterItem } from './characterItem'
import { ItemType, ItemRarity } from '#application/enums'

@Entity()
export class Item extends BaseEntity {
  @PrimaryKey()
  id!: string

  @Property()
  name!: string

  @Property({ nullable: true })
  description?: string

  @Enum(() => ItemType)
  itemType!: ItemType

  @Property()
  stackable = false

  @Enum(() => ItemRarity)
  rarity: ItemRarity = ItemRarity.COMMON

  @ManyToOne(() => Sprite, { nullable: true })
  sprite?: Sprite

  @Property()
  createdAt = new Date()

  @Property()
  updatedAt = new Date()

  @OneToMany(() => CharacterItem, (characterItem) => characterItem.item)
  characters = new Collection<CharacterItem>(this)
}
