import { gameLogger } from '#application/logger'
import WorldRepository from '#repositories/worldRepository'
import { World } from '#entities/world'

class WorldService {
  async update(worldData: Partial<World>): Promise<boolean> {
    try {
      let world = await WorldRepository.getFirst()
      if (!world) {
        world = new World()

        await world.save()
      }

      world.date = worldData.date || new Date()

      if (worldData.isRainEnabled) {
        world.isRainEnabled = worldData.isRainEnabled
      }
      if (worldData.rainPercentage) {
        world.rainPercentage = worldData.rainPercentage
      }
      if (worldData.isFogEnabled) {
        world.isFogEnabled = worldData.isFogEnabled
      }
      if (worldData.fogDensity) {
        world.fogDensity = worldData.fogDensity
      }

      console.log(world.date)

      await world.update()

      return true
    } catch (error: any) {
      gameLogger.error(`Failed to update world: ${error instanceof Error ? error.message : String(error)}`)
      return false
    }
  }
}

export default new WorldService()
