import { ExtendedCharacter, TSocket } from '#application/types'
import prisma from '#application/prisma'
import { ZoneEventTileTeleport } from '@prisma/client'
import { Server } from 'socket.io'
import ZoneManager from '#managers/zoneManager'
import { gameLogger } from '#application/logger'

export class ZoneEventTileService {
  public async handleTeleport(io: Server, socket: TSocket, character: ExtendedCharacter, teleport: ZoneEventTileTeleport): Promise<void> {
    if (teleport.toZoneId === character.zoneId) return

    const loadedZone = ZoneManager.getZoneById(teleport.toZoneId)
    if (!loadedZone) {
      gameLogger.error('zone:character:join error', 'Loaded zone not found')
      return
    }

    const zone = loadedZone.getZone()

    const oldZoneId = character.zoneId
    const newZoneId = teleport.toZoneId

    // Update character in database
    await prisma.character.update({
      where: { id: character.id },
      data: {
        zoneId: newZoneId,
        positionX: teleport.toPositionX,
        positionY: teleport.toPositionY,
        rotation: teleport.toRotation
      }
    })

    // Update local character object
    character.zoneId = newZoneId
    character.rotation = teleport.toRotation
    character.positionX = teleport.toPositionX
    character.positionY = teleport.toPositionY
    character.isMoving = false

    // Remove and add character to new zone
    await loadedZone.removeCharacter(character.id)
    loadedZone.addCharacter(character)

    // Emit events
    io.to(oldZoneId.toString()).emit('zone:character:leave', character.id)
    io.to(newZoneId.toString()).emit('zone:character:join', character)

    // Update socket rooms
    socket.leave(oldZoneId.toString())
    socket.join(newZoneId.toString())

    // Send teleport information to the client
    socket.emit('zone:character:teleport', {
      zone,
      characters: loadedZone.getCharactersInZone()
    })
  }
}
